/*Copyright (C) 2010-2023 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MultipleKeyBinder.hpp"
#include "Engines/Resources/TextureManager.hpp"
#include "Engines/Sound/SoundEngine.hpp"
#include "Player/PlayerKeys.hpp"

#include "GUI/Widget/TextButton.hpp"
#include "Tools/ScaledPixel.hpp"

MultipleKeyBinder::MultipleKeyBinder() : Tab(), m_deviceID(DEVICE_KEYBOARD) {}

MultipleKeyBinder::MultipleKeyBinder(int deviceID,
                                     const PlayerKeys &initialKeys,
                                     TextureManager &manager,
                                     Font &labelFont,
                                     SoundEngine &soundEngine,
                                     float viewWidth) :
Tab(ScaledPixel{viewWidth}(250.f),
    ScaledPixel{viewWidth}(200.f),
    Color(40, 83, 141)),
m_deviceID(deviceID)
{
    const ScaledPixel sp(viewWidth);

    Sprite chooser_default(manager.getTexture("gfx/ui/ui.png"),
                           Box(0.f, 0.f, 240.f / 512.f, 60.f / 512.f)),
    chooser_hover(manager.getTexture("gfx/ui/ui.png"),
                  Box(0.f, 64.f / 512.f, 240.f / 512.f, 60.f / 512.f));

    chooser_default.setScale(sp(.4f, false));
    chooser_hover.setScale(sp(.4f, false));

    //Key binders
    const float xBase = sp(25.f), xBaseLabel = sp(100.f), yBase = sp(7.f);
    const float spacing =
        chooser_default.getHeight() * chooser_default.getYScale() + sp(3.f);

    m_punch =
        KeyLabel("Punch:", labelFont,
                 KeyBinder(initialKeys.getPunchKey()),
                 xBase, xBaseLabel, yBase, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_kick =
        KeyLabel("Kick:", labelFont,
                 KeyBinder(initialKeys.getKickKey()),
                 xBase, xBaseLabel, yBase + spacing, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_throwWeapon =
        KeyLabel("Weapon:", labelFont,
                 KeyBinder(initialKeys.getThrowWeaponKey()),
                 xBase, xBaseLabel, yBase + spacing * 2.f, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_jump =
        KeyLabel("Jump:", labelFont,
                 KeyBinder(initialKeys.getJumpKey()),
                 xBase, xBaseLabel, yBase + spacing * 3.f, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_moveLeft =
        KeyLabel("Move left:", labelFont,
                 KeyBinder(initialKeys.getMoveLeftKey()),
                 xBase, xBaseLabel, yBase + spacing * 4.f, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_moveRight =
        KeyLabel("Move right:", labelFont,
                 KeyBinder(initialKeys.getMoveRightKey()),
                 xBase, xBaseLabel, yBase + spacing * 5.f, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");

    m_crouch =
        KeyLabel("Crouch:", labelFont,
                 KeyBinder(initialKeys.getCrouchKey()),
                 xBase, xBaseLabel, yBase + spacing * 6.f, chooser_default,
                 chooser_hover, soundEngine, "audio/ui/menuFocus.ogg",
                 "audio/ui/menuClick.ogg");
}

void MultipleKeyBinder::draw()
{
    //Update matrix
    Drawable::pushMatrix();
    Drawable::updateMatrix();

    m_background.draw();

    //Draw binders
    m_punch.draw();
    m_kick.draw();
    m_throwWeapon.draw();
    m_jump.draw();
    m_moveLeft.draw();
    m_moveRight.draw();
    m_crouch.draw();

    Drawable::popMatrix();
}

bool MultipleKeyBinder::mouseRelease(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);

    m_punch.binder.mouseRelease(localMouse);
    m_kick.binder.mouseRelease(localMouse);
    m_throwWeapon.binder.mouseRelease(localMouse);
    m_jump.binder.mouseRelease(localMouse);
    m_moveLeft.binder.mouseRelease(localMouse);
    m_moveRight.binder.mouseRelease(localMouse);
    m_crouch.binder.mouseRelease(localMouse);

    return Widget::contains(localMouse);
}

bool MultipleKeyBinder::mouseClick(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    return Widget::contains(localMouse);
}

bool MultipleKeyBinder::mouseMove(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    return Widget::contains(localMouse);
}

void MultipleKeyBinder::keyDown(const Key &key, int deviceID)
{
    if ((key.isFromKeyboard() && m_deviceID == DEVICE_KEYBOARD) ||
        (key.isFromJoystick() && m_deviceID == deviceID))
    {
        m_punch.binder.keyDown(key);
        m_kick.binder.keyDown(key);
        m_throwWeapon.binder.keyDown(key);
        m_jump.binder.keyDown(key);
        m_moveLeft.binder.keyDown(key);
        m_moveRight.binder.keyDown(key);
        m_crouch.binder.keyDown(key);
    }
}

void MultipleKeyBinder::setFocus(bool focused)
{
    Widget::setFocus(focused);

    if (!focused)
    {
        m_punch.binder.setFocus(false);
        m_kick.binder.setFocus(false);
        m_throwWeapon.binder.setFocus(false);
        m_jump.binder.setFocus(false);
        m_moveLeft.binder.setFocus(false);
        m_moveRight.binder.setFocus(false);
        m_crouch.binder.setFocus(false);
    }
}

void MultipleKeyBinder::resetKeys(const PlayerKeys &keys)
{
    m_punch.binder.setKey(keys.getPunchKey());
    m_kick.binder.setKey(keys.getKickKey());
    m_throwWeapon.binder.setKey(keys.getThrowWeaponKey());
    m_jump.binder.setKey(keys.getJumpKey());
    m_moveLeft.binder.setKey(keys.getMoveLeftKey());
    m_moveRight.binder.setKey(keys.getMoveRightKey());
    m_crouch.binder.setKey(keys.getCrouchKey());
}

PlayerKeys MultipleKeyBinder::getKeys() const
{
    return PlayerKeys(m_punch.binder.getKey(),
                      m_kick.binder.getKey(),
                      m_throwWeapon.binder.getKey(),
                      m_jump.binder.getKey(),
                      m_moveLeft.binder.getKey(),
                      m_moveRight.binder.getKey(),
                      m_crouch.binder.getKey());
}
