/*Copyright (C) 2010-2023 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_FORMAT
#define DEF_FORMAT

#include <sstream>
#include <vector>
#include <string>

namespace Format
{
    namespace
    {
        /// Converts arbitrarily typed arguments to std::string if possible
        /// and adds it to a resulting std::vector<std::string>
        template <typename...>
        void addToVect(std::vector<std::string> &/*target*/) {}

        template <typename T, typename... Ts>
        void addToVect(std::vector<std::string> &target, T&& head, Ts&&... tail)
        {
            std::stringstream sstream;
            sstream << head;
            target.push_back(sstream.str());

            addToVect(target, tail...);
        }
    }

    template <typename... Ts>
    inline std::string format(const std::string &sentence, Ts&&... args)
    {
        std::vector<std::string> strArgs;
        strArgs.reserve(sizeof...(Ts));
        addToVect(strArgs, args...);

        std::string formatted(sentence);

        unsigned int argsIndex = 0;
        std::size_t findIndex = 0;
        bool browsed = false;
        while (!browsed)
        {
            if (argsIndex >= strArgs.size())
                browsed = true;
            else
            {
                findIndex = formatted.find("{}", findIndex);
                if (findIndex == std::string::npos)
                    browsed = true;
                else
                {
                    formatted.replace(findIndex, 2, strArgs[argsIndex]);
                    findIndex += strArgs[argsIndex].length();
                    argsIndex++;
                }
            }
        }

        return formatted;
    }

    /// Pads \p str by appending space characters at its end until it reaches
    /// the requested size \p length, does nothing is length <= str.size()
    inline std::string pad(const std::string &str, std::size_t length)
    {
        return (str.length() >= length) ? str :
            str + std::string(length - str.length(), ' ');
    }

    /// Formats the description of a command line option in a --help text
    /// Returns \p label, padded to reach \p labelLength,
    /// followed by the description
    inline std::string column(const std::string &label,
                              const std::string &description,
                              std::size_t labelLength)
    {
        return pad(label, labelLength) + description;
    }
}

#endif
